/*
 * Copyright (c) 2015 Accorto, Inc. All Rights Reserved
 * License: GPLv3   http://www.gnu.org/licenses/gpl-3.0.txt
 * License options+support:  https://www.lightningdart.com
 */

part of lightning_dart.demo;

/**
 * Demo Feature Mgt
 */
abstract class DemoFeature extends LComponent {

  static const String SLDS_URL = "https://www.lightningdesignsystem.com/components";
  static const String SLDS_TXT = "SLDS reference";

  final DivElement element = new DivElement();

  final String id;
  final String label;
  final String sldsPath;

  /**
   * Feature
   */
  DemoFeature(String this.id, String this.label, String description, {String this.sldsPath}) {
    element.id = id;
    DivElement hdr = new DivElement()
      ..classes.addAll([LMargin.C_TOP__X_LARGE, LTheme.C_BOX, LTheme.C_THEME__SHADE]);
    HeadingElement h2 = new HeadingElement.h2()
      ..classes.add(LText.C_TEXT_HEADING__MEDIUM)
      ..text = label;
    hdr.append(h2);

    ParagraphElement descr = new ParagraphElement()
      ..text = description;
    hdr.append(descr);

    ParagraphElement ref = new ParagraphElement()
      ..classes.add(LText.C_TEXT_BODY__SMALL)
      ..text = "See: "
      ..append(createSldsLink());
    hdr.append(ref);

    UListElement ul = new UListElement()
      ..classes.addAll([LList.C_LIST__VERTICAL, LMargin.C_TOP__SMALL]);
    for (DivElement option in options) {
      LIElement li = new LIElement()
        ..classes.add(LList.C_LIST__ITEM);
      li.append(option);
      ul.append(li);
    }
    // element.append(new HRElement());
    hdr.append(ul);
    element.append(hdr);


    //
    LTab tab = new LTab(idPrefix: id);
    DivElement wrapper = new DivElement()
      ..classes.add(LMargin.C_LEFT__SMALL);
    wrapper.append(tab.element);
    element.append(wrapper);

    //
    Element sc = tab.addTab("Small", name: "s");
    sc.style.width = "480px";
    sc.style.border = "1px solid lightgray";
    Element mc = tab.addTab("Medium", name: "m");
    mc.style.width = "768px";
    mc.style.border = "1px solid gray";
    Element lc = tab.addTab("Large", name: "l");
    lc.style.width = "1024px";
    lc.style.border = "1px solid black";
    Element xc = tab.addTab("Fluid", name: "x");
    xc.style.width = "100%";
    xc.style.border = "1px solid black";

    Element dc = tab.addTab("Source", name: "d");
    ParagraphElement src = new ParagraphElement()
      ..classes.add(LText.C_TEXT_BODY__SMALL)
      ..style.whiteSpace = "pre"
      ..text = source;
    dc.append(src);

    tab.onTabChanged.listen(onTabChanged); // init display on tab add
    tab.selectTabByPos(3); // fluid
  } // DemoFeature

  /// Link to Feature
  void toc(Element toc) {
    DivElement col = new DivElement()
      ..classes.addAll([LGrid.C_COL, LMargin.C_AROUND__SMALL])
      ..style.minWidth = "10rem";
    toc.append(col);
    col.append(new AnchorElement(href: "#${id}")
      ..text = label);
  }

  AnchorElement createSldsLink() {
    String path = sldsPath;
    if (path == null)
      path = id;
    return new AnchorElement(href: "${SLDS_URL}/$path")
      ..target = "slds"
      ..text = SLDS_TXT;
  }


  LComponent get content => contentNIY;
  String get source => sourceNIY;
  List<DivElement> get options => [];


  // Tab changed
  void onTabChanged(LTab tab){
    if (_theContent == null)
      _theContent = content;
    else
      _theContent.element.remove();
    //
    _theCurrentElement = tab.currentContent;
    _theCurrentElement.append(_theContent.element);
    //
    // window.location.hash = id;
  }
  LComponent _theContent;
  Element _theCurrentElement;

  /*
  LComponent get content {
    CDiv div = new CDiv();
    return div;
  }
  String get source {
    return '''
    ''';
  }
  */

  //option changed
  void optionChanged() {
    _theContent.element.remove();
    _theContent = content; // new
    if (_theCurrentElement != null)
      _theCurrentElement.append(_theContent.element);
  }


  LComponent get contentNIY {
    return new CDiv()
        ..text = "Not Implemented Yet";
  }
  String get sourceNIY {
    return "Not Implemented Yet";
  }

  /// Generate List items
  List<ListItem> generateListItems(int count,
      {String prefix: "Item", int increment: 4, bool iconLeft: false}) {
    List<ListItem> list = new List<ListItem>();
    int no = 1;
    for (int i = 0; i < count; i++) {
      String label = "${prefix} ${no}";
      String value = "${prefix.toLowerCase()}${no}";

      LIcon leftIcon = null;
      if (iconLeft) {
        leftIcon = new LIconCustom("custom-${no}");
      }
      DOption option = new DOption()
        ..id = value
        ..value = value
        ..label = label;
      ListItem li = new ListItem(option, leftIcon:leftIcon);
      list.add(li);
      no += increment;
    }
    return list;
  }

} // DemoFeature
